@@ -1,4 +1,5 @@
 import email
+from email.header import decode_header
 from email.message import Message as EmailMessage
 from email.utils import formatdate, parseaddr
 from email.encoders import encode_base64
@@ -16,7 +17,7 @@ except ImportError:
 
 from django.conf import settings
 from django.core.mail.message import make_msgid
-from django.core.files import File
+from django.core.files.base import File, ContentFile
 from django.db import models
 from django_mailbox.transports import Pop3Transport, ImapTransport,\
     MaildirTransport, MboxTransport, BabylTransport, MHTransport, \
@@ -24,11 +25,7 @@ from django_mailbox.transports import Pop3Transport, ImapTransport,\
 from django_mailbox.signals import message_received
 import six
 
-SKIPPED_EXTENSIONS = getattr(
-    settings,
-    'DJANGO_MAILBOX_SKIPPED_EXTENSIONS',
-    ['.p7s']
-)
+
 STRIP_UNALLOWED_MIMETYPES = getattr(
     settings,
     'DJANGO_MAILBOX_STRIP_UNALLOWED_MIMETYPES',
@@ -227,9 +224,10 @@ class Mailbox(models.Model):
                 _, extension = os.path.splitext(filename)
 
             attachment = MessageAttachment()
+            
             attachment.document.save(
                 uuid.uuid4().hex + extension,
-                File(six.BytesIO(msg.get_payload(decode=True)))
+                ContentFile(six.BytesIO(msg.get_payload(decode=True)).getvalue())
             )
             attachment.message = record
             for key, value in msg.items():
@@ -250,10 +248,14 @@ class Mailbox(models.Model):
     def _process_message(self, message):
         msg = Message()
         msg.mailbox = self
-        msg.subject = message['subject'][0:255]
-        msg.message_id = message['message-id'][0:255]
-        msg.from_header = message['from']
-        msg.to_header = message['to']
+        if 'subject' in message:
+            msg.subject = message['subject'][0:255]
+        if 'message-id' in message:
+            msg.message_id = message['message-id'][0:255]
+        if 'from' in message:
+            msg.from_header = message['from']
+        if 'to' in message:
+            msg.to_header = message['to']
         msg.save()
         message = self._get_dehydrated_message(message, msg)
         msg.body = message.as_string()
@@ -351,24 +353,27 @@ class Message(models.Model):
         separate fields.
 
         """
-        if self.outgoing:
-            return self.to_addresses()
-        else:
-            return self.from_addresses()
+        addresses = []
+        addresses = self.to_addresses +  self.from_address
+        return addresses
 
     @property
     def from_address(self):
-        return parseaddr(self.from_header)[1]
+        if self.from_header:
+            return [parseaddr(self.from_header)[1].lower()]
+        else:
+            return []
 
     @property
     def to_addresses(self):
         addresses = []
         for address in self.to_header.split(','):
-            addresses.append(
-                parseaddr(
-                    address
-                )[1]
-            )
+            if address:
+                addresses.append(
+                    parseaddr(
+                        address
+                    )[1].lower()
+                )
         return addresses
 
     def reply(self, message):
@@ -406,7 +411,11 @@ class Message(models.Model):
                     part.get_content_maintype() == 'text'
                     and part.get_content_subtype() == 'plain'
                 ):
-                    body = body + part.get_payload()
+                    charset = part.get_content_charset()
+                    this_part = part.get_payload(decode=True)
+                    if charset:
+                        this_part = this_part.decode(charset)
+                    body += this_part
             return body
 
         return get_body_from_message(
@@ -471,9 +480,9 @@ class Message(models.Model):
     def get_email_object(self):
         """ Returns an `email.message.Message` instance for this message.
 
-        Note: Python 2.6's version of email.message_from_string requires
-        bytes and will incorrectly create an e-mail object if a unicode
-        object is passed-in.
+        Note: Python 2's version of email.message.fromstring on some platforms
+        requires bytes and will incorrectly create an e-mail object if a
+        unicode object is passed-in.
 
         Note that in reality 7-bit ascii *should* be an acceptable encoding
         here per RFC-2822 (2.1), but given that `message_from_string` really
@@ -484,7 +493,7 @@ class Message(models.Model):
 
         """
         body = self.body
-        if sys.version_info < (2, 7):
+        if sys.version_info < (3, 0):
             body = body.encode('utf-8')
         return self._rehydrate(
             email.message_from_string(body)
@@ -518,7 +527,7 @@ class MessageAttachment(models.Model):
         headers = self.headers
         if headers is None:
             return EmailMessage()
-        if sys.version_info < (2, 7):
+        if sys.version_info < (3, 0):
             headers = headers.encode('utf-8')
         return email.message_from_string(headers)
 
@@ -536,13 +545,23 @@ class MessageAttachment(models.Model):
         self._set_dehydrated_headers(rehydrated)
 
     def get_filename(self):
-        return self._get_rehydrated_headers().get_filename()
+        file_name = self._get_rehydrated_headers().get_filename()
+        if file_name:
+            encoded_subject, encoding = decode_header(file_name)[0]
+            if encoding:
+                encoded_subject = encoded_subject.decode(encoding)
+            return encoded_subject
+        else:
+            return None
 
     def items(self):
         return self._get_rehydrated_headers().items()
 
     def __getitem__(self, name):
-        return self._get_rehydrated_headers()[name]
+        value = self._get_rehydrated_headers()[name]
+        if value is None:
+            raise KeyError('Header %s does not exist' % name)
+        return value
 
     def __unicode__(self):
         return self.document.url
